// vincia21.cc is a part of the VINCIA event generator.
// Copyright (C) 2018 Peter Skands.
// VINCIA is licenced under the GNU GPL version 2, see COPYING for details.
// Please respect the MCnet Guidelines, see GUIDELINES for details.

// This is a simple test program (adapted from the PYTHIA example main05.cc)
// It studies gg->gg at the LHC, and is intended to test the interoperation
// of ISR and FSR in VINCIA.

#include "Vincia/Vincia.h"
using namespace Pythia8;
using namespace Vincia;

int main() {

  // Number of events, generated and listed ones.
  int nEvent    = 1000;
  int nListJets = 5;

  // Generator. LHC process and output selection. Initialization.
  Pythia pythia;
  VinciaPlugin vincia(&pythia);
  pythia.readString("Beams:eCM = 7000.");
  pythia.readString("HardQCD:gg2gg = on");
  pythia.readString("Vincia:FSR = on");
  pythia.readString("Vincia:ISR = on");
  pythia.readString("PhaseSpace:pTHatMin = 200.");
  pythia.readString("Next:numberShowInfo = 0");
  pythia.readString("Next:numberShowProcess = 5");
  pythia.readString("Next:numberShowEvent = 5");
  // Switch off MPI and hadronization 
  pythia.readString("PartonLevel:MPI = off");
  pythia.readString("HadronLevel:all = off");

  // VINCIA settings
  // Debug level and MECs
  pythia.readString("Vincia:verbose = 1");
  pythia.readString("Vincia:matchingLO = 0");

  // Initialize
  vincia.init();

  // Set up SlowJet jet finder, with anti-kT clustering
  // and pion mass assumed for non-photons
  double etaMax   = 4.;
  double radius   = 0.7;
  double pTjetMin = 10.;
  // Exclude neutrinos (and other invisible) from study.
  int    nSel     = 2;
  SlowJet slowJet( -1, radius, pTjetMin, etaMax, nSel, 1);

  // Histograms. Note similarity in names, even when the two jet finders
  // do not calculate identically the same property (pT vs. ET, y vs. eta).
  Hist nJets("number of jets", 50, -0.5, 49.5);
  Hist eTjets("pT for jets", 100, 0., 500.);
  Hist etaJets("y for jets", 100, -5., 5.);
  Hist phiJets("phi for jets", 100, -M_PI, M_PI);
  Hist distJets("R distance between jets", 100, 0., 10.);
  Hist eTdiff("pT difference", 100, -100., 400.);

  // Begin event loop. Generate event. Skip if error.
  for (int iEvent = 0; iEvent < nEvent; ++iEvent) {
    if (!pythia.next()) continue;

    // Analyze SlowJet jet properties. List first few.
    /*
    slowJet. analyze( pythia.event );
    if (iEvent < nListJets) slowJet.list();

    // Fill SlowJet inclusive jet distributions.
    nJets.fill( slowJet.sizeJet() );
    for (int i = 0; i < slowJet.sizeJet(); ++i) {
      eTjets.fill( slowJet.pT(i) );
      etaJets.fill( slowJet.y(i) );
      phiJets.fill( slowJet.phi(i) );
    }

    // Fill SlowJet distance between jets.
    for (int i = 0; i < slowJet.sizeJet() - 1; ++i)
    for (int j = i +1; j < slowJet.sizeJet(); ++j) {
      double dEta = slowJet.y(i) - slowJet.y(j);
      double dPhi = abs( slowJet.phi(i) - slowJet.phi(j) );
      if (dPhi > M_PI) dPhi = 2. * M_PI - dPhi;
      double dR = sqrt( pow2(dEta) + pow2(dPhi) );
      distJets.fill( dR );
    }

    // Fill SlowJet pT-difference between jets (to check ordering of list).
    for (int i = 1; i < slowJet.sizeJet(); ++i)
      eTdiff.fill( slowJet.pT(i-1) - slowJet.pT(i) );
    */
    
  // End of event loop. Statistics. Histograms.
  }
  pythia.stat();
  //cout << nJets << eTjets << etaJets << phiJets << distJets << eTdiff;

  // Done.
  return 0;
}
