// Merging.h is a part of the VINCIA plug-in to the PYTHIA 8 generator.
// Copyright (C) 2018 P. Skands.
// VINCIA is licenced under the GNU GPL version 2.
// Please respect the MCnet Guidelines for Event Generator Authors and Users, 
//               http://www.montecarlonet.org/GUIDELINES

// This file contains header information for the VinciaMerging class.
// Most of the code is copied from the Pythia Merging class.

#ifndef Vincia_Merging_H
#define Vincia_Merging_H

#include "Pythia8/Merging.h"
#include "Vincia/MergingHooks.h"
#include "Vincia/History.h"

using namespace Pythia8;

namespace Vincia {

  // Forward declarations.
  class TimeAntShower;
  class SpaceAntShower;
  class VinciaMergingHooks;

  class VinciaMerging : public Pythia8::Merging {

  public:

  // Constructor.
  VinciaMerging() { 
    settingsPtr         = 0;
    infoPtr             = 0;
    particleDataPtr     = 0;
    rndmPtr             = 0;
    beamAPtr            = 0;
    beamBPtr            = 0;
    trialPartonLevelPtr = 0;
    mergingHooksPtr     = 0;
    vcMergingHooksPtr   = 0;
    timesAntShowerPtr   = 0;
    spaceAntShowerPtr   = 0;
    vinComPtr           = 0;
  }

  // Destructor.
  ~VinciaMerging() {}

  // Initialize.
  void init();

  // Set the necessary Vincia pointers.
  void setVinciaPtrs(VinciaMergingHooks* mPtrIn, TimeAntShower* tPtrIn,
    SpaceAntShower* sPtrIn, VinciaCommon* cPtrIn) {
    vcMergingHooksPtr = mPtrIn;
    timesAntShowerPtr = tPtrIn;
    spaceAntShowerPtr = sPtrIn;
    vinComPtr         = cPtrIn;
  }

  // Main function to perform the merging.
  int mergeProcess(Event& process);

  // Function to apply the merging scale cut on an input event.
  bool cutOnProcess(Event& process);

  // Print statistics.
  void statistics();

  private:

  // Pointer to Vincia objects.
  VinciaMergingHooks* vcMergingHooksPtr;
  TimeAntShower*      timesAntShowerPtr;
  SpaceAntShower*     spaceAntShowerPtr;
  VinciaCommon*       vinComPtr;

  // Minimal value found for the merging scale in events.
  double tmsNowMin;
  static const double TMSMISMATCH;

  };

}

#endif
