// vincia31_uxu_gg.cc is a part of the VINCIA plug-in to the PYTHIA 8 generator.
// Copyright (C) 2018 P. Skands.
// VINCIA is licenced under the GNU GPL version 2.
// Please respect the MCnet Guidelines for Event Generator Authors and Users,
//               http://www.montecarlonet.org/GUIDELINES

// This file was created on Tue 16 May 2017 14:36:53 AEST by ./convertMG5.sh
// VINCIA C++ wrapper for vincia31_uxu_gg.f

#include "VinciaMG5/vincia31_uxu_gg.h"

namespace Vincia {

//====================================================================

// Wrapper for the MG5 FORTRAN subroutine itself

extern "C" {
  extern void shelvincia31_uxu_gg_(int *cMode, int *ICOL, double P1[4][4], int *HEL, double* ANS);
}

//--------------------------------------------------------------------

// initHelMap(): create map from vector of helicities to IHEL number

void MG5vincia31_uxu_gg::initHelMap() {
  // Hardcoded helicity map extracted from .f file
  vector<int> h; h.resize(4);
  h[0]=-1;  h[1]=1;  h[2]=-1;  h[3]=-1;  hMapSav[h] = 1;
  h[0]=-1;  h[1]=1;  h[2]=-1;  h[3]=1;  hMapSav[h] = 2;
  h[0]=-1;  h[1]=1;  h[2]=1;  h[3]=-1;  hMapSav[h] = 3;
  h[0]=-1;  h[1]=1;  h[2]=1;  h[3]=1;  hMapSav[h] = 4;
  h[0]=-1;  h[1]=-1;  h[2]=-1;  h[3]=-1;  hMapSav[h] = 5;
  h[0]=-1;  h[1]=-1;  h[2]=-1;  h[3]=1;  hMapSav[h] = 6;
  h[0]=-1;  h[1]=-1;  h[2]=1;  h[3]=-1;  hMapSav[h] = 7;
  h[0]=-1;  h[1]=-1;  h[2]=1;  h[3]=1;  hMapSav[h] = 8;
  h[0]=1;  h[1]=1;  h[2]=-1;  h[3]=-1;  hMapSav[h] = 9;
  h[0]=1;  h[1]=1;  h[2]=-1;  h[3]=1;  hMapSav[h] = 10;
  h[0]=1;  h[1]=1;  h[2]=1;  h[3]=-1;  hMapSav[h] = 11;
  h[0]=1;  h[1]=1;  h[2]=1;  h[3]=1;  hMapSav[h] = 12;
  h[0]=1;  h[1]=-1;  h[2]=-1;  h[3]=-1;  hMapSav[h] = 13;
  h[0]=1;  h[1]=-1;  h[2]=-1;  h[3]=1;  hMapSav[h] = 14;
  h[0]=1;  h[1]=-1;  h[2]=1;  h[3]=-1;  hMapSav[h] = 15;
  h[0]=1;  h[1]=-1;  h[2]=1;  h[3]=1;  hMapSav[h] = 16;
}

//--------------------------------------------------------------------

// initColMap(): create map from vector of MG indices to ICOL numbers

void MG5vincia31_uxu_gg::initColMap() {
  // Hardcoded colour-index map extracted from .f file
  // Colour traces extracted from MG5 fortran file
  vector< vector<int> > traces; traces.resize(0);
  vector<int> traceNow;
  // ICOL 1
  traceNow.resize(0); traces.resize(0);
  traceNow.push_back(3);
  traceNow.push_back(4);
  traceNow.push_back(1);
  traceNow.push_back(2);
  traces.push_back(traceNow);
  cMapSav[traces[0]]=1;
  // ICOL 2
  traceNow.resize(0); traces.resize(0);
  traceNow.push_back(4);
  traceNow.push_back(3);
  traceNow.push_back(1);
  traceNow.push_back(2);
  traces.push_back(traceNow);
  cMapSav[traces[0]]=2;
}

//------------------------------------------------------------------

// Return PDG ID label (main identifier for this process in Vincia)

 vector< vector<int> > MG5vincia31_uxu_gg::processList() {
  vector<int> label; label.resize(0);
  label.push_back(-2);
  label.push_back(2);
  label.push_back(21);
  label.push_back(21);
  // Sanity check
  if (nLegs() != (int)label.size())
    cout<<"  MG5${procName}::processLabel(): ERROR! inconsistent number of legs"<<endl;
  vector< vector<int> > list;
  list.push_back(label);
  return list;
}

//------------------------------------------------------------------

// Wrapper for MG5 ME, with variables set by the other methods

double MG5vincia31_uxu_gg::getME2(int mcMode) {
  // Call wrapped matrix element
  double ans;
  shelvincia31_uxu_gg_(&mcMode, &icolSav, pSav, &ihelSav, &ans);
  return ans;
}

} // End namespace Vincia

